puts -nonewline "+++ loading Tcl init script: "

namespace import ::tcl::mathop::*

;# console colors
set color(header) {set-text-color 1 0.75 0.5}
set color(text) {set-text-color 0.8 1 0.8}
set color(error) {set-text-color 1 0.6 0.6}
set color(fine) {set-text-color 0.6 1 0.6}

set src_body {{name} {
	source ~/.din/$name.tcl
}}

proc src {*}$src_body

proc midimaker {what} {
	proc midi${what}all [list [list op $what]] {
		for { set i 0; set j [midi num] } { $i < $j } {incr i} {
			echo [midi $op $i]
		}
	}
}

midimaker open ;# gives midiopenall command
midimaker close ;# gives midicloseall command

proc tap_bpm_changed {name element op} {
	;# tap bpm changed
	upvar taptarget target tapbpm bpm
	echo "tap bpm: $bpm"
	foreach i $target {
		set-bpm $i $bpm reset
	}
}

trace add variable tapbpm write tap_bpm_changed

proc lsdotdin {{ext *} {orient h} } {
	set mapping [list .$ext {}]
	if {$orient eq "v" || $orient eq "vertical"} {lappend mapping { } \n}
	string map $mapping [lsort [glob -tails -directory ~/.din *.$ext]]
}

;# list_resonators
set lsr_body {{{orient h} {ext res}} {lsdotdin $ext $orient}}
proc ls-resonators {*}$lsr_body
proc lsr {*}$lsr_body

proc delete-disabled-resonators {} {
	save-resonators . 1
	load-resonators .
}

proc bad_sub_command {c} {
	upvar $c cmds
	set err "bad sub-command. should be: "
	foreach i [lrange $cmds 0 end-1] {
		append err "$i, "
	}
	append err "or [lindex $cmds end]"
}

proc exec_sub_command {ucmds uactions ucmd uargs} {
	upvar $ucmds cmds $uactions actions $ucmd cmd $uargs args
	set j 0
	foreach i $cmds {
		if {$cmd eq $i} {
			return [{*}[lindex $actions $j] {*}$args]
		}
		incr j
	}
	bad_sub_command cmds
}

proc find-scale {pattern} {
	lsort [lsearch -all -inline [ls-scale] $pattern]
}

proc tuning {cmd args} { ;# tuning command
	set cmds {list set get}
	set actions {"lsdotdin tuning" "sv tuning" "gv tuning"}
	exec_sub_command cmds actions cmd args
}

proc bpm-set {target value args} {
	foreach i $target {
		set-bpm $i $value
	}
}

proc bpm-get {target args} {
	foreach i $target {
			append result "[get-bpm $i] "
	}
	return $result
}

proc bpm-reset {target args} {
	foreach i $target {
		set-bpm $i [get-bpm $i] reset
	}
}

proc bpm-style-set {target style args} {
	foreach i $target {
		set-style $i $style
	}
}

proc bpm-style-get {target args} {
	foreach i $target {
			append result "[get-style $i] "
	}
	return $result
}

source ~/.din/help.tcl
source ~/.din/midimap.tcl

puts "done. +++"
